<?php
/**
 * EventApiModel class
 *
 * PHP version 5
 *
 * @category ADPOI
 * @package  Api.controller
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
/**
 * EventApiModel is the model class
 * for managing Event records
 *
 * @category ADPOI
 * @package  Api.controller
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
class EventApiModel extends RestApiModel
{
    const DATE_FORMAT = 'yyyy-MM-dd HH:mm';
    const MEMBERS_SEPARATOR = ',';
    const TERM_MAX_LENGTH = 128;
    const DEFAULT_PAGE_SIZE = 10;
    
    /**
     * Event's id
     * @var integer
     */
    public $id;
    
    /**
     * Poi's id
     * @var integer
     */
    public $poi_id;

    /**
     * Event creator identifier
     * @var integer
     */
    public $creator_id;
    
    /**
     * User id for searching events
     * @var integer
     */
    public $user_id;
    
    /**
     * Comma separated user id
     * @var string
     */
    public $members;
    
    /**
     * Events' name
     * @var string
     */
    
    public $name;
    
    /**
     * Event's start time
     * @var datetime
     */
    public $start_time;
    
    /**
     * Event's end time
     * @var datetime
     */
    public $end_time;
    
    /**
     * Event's top border of start time
     * @var datetime
     */
    public $to;
    
    /**
     * Event's lower border of start time
     * @var datetime
     */
    public $from;
    
    /**
     * Event's description
     * @var string
     */
    public $description;
        
    /**
     * Event's state name
     * @var string
     */
    public $state;
    
    /**
     * Count of events on page
     * @var integer
     */
    public $count;
    
    /**
     * Number of event's page
     * @var integer
     */
    public $page;
    
    /**
     * Query request for seach event
     * @var string
     */
    public $term;
    
    /**
     * Return the validation rulles
     * 
     * @return array validation rules to be applied 
     * when {@link validate()} is called.
     * @see CModel::rules()
     */
    public function rules()
    {
        $base = parent::rules();
        $rules = array(
            array('id', 'required', 'on' => 'update, delete, show'),
            array('id', 'numerical', 'integerOnly' => true, 'on' => 'update, delete, show'),
            array('id', 'exist', 'className' => 'Event', 'attributeName' => 'id', 'on' => 'update, delete, show'),
            array('poi_id', 'numerical',  'integerOnly' => true, 'on' => 'create'),
            array('poi_id', 'exist', 'className' => 'Poi', 'attributeName' => 'id', 'on' => 'create'),
            array('name, start_time, end_time', 'required', 'on' => 'create'),
            array('name', 'length', 'max' => Event::NAME_MAX_LENGTH, 'on' => 'create, update'),
            array('start_time, end_time', 'date', 'format' => self::DATE_FORMAT, 'on' => 'create, update'),
            array('from', 'default', 'value' => Yii::app()->formatter->datetimeWithoutSeconds(time()),'on' => 'list'),
            array('to', 'default', 'value' => Yii::app()->formatter->datetimeWithoutSeconds(time() + EventSearch::DEFAULT_TIME_PERIOD),'on' => 'list'),
            array('from, to', 'date', 'format' => EventApiModel::DATE_FORMAT, 'on' => 'list'),
            array('description', 'length', 'max' => Event::DESCRIPTION_MAX_LENGTH, 'on' => 'create, update'),
            array('start_time', 'compare', 'compareValue' => 'end_time', 'operator' => '<', 'on' => 'create, update'),
            array('count, page', 'numerical', 'integerOnly' => true, 'min' => 1, 'on' => 'list'),
            array('page', 'default', 'value' => 1, 'on' => 'list'),
            array('count', 'default', 'value' => self::DEFAULT_PAGE_SIZE, 'on' => 'list'),
            array('term', 'length', 'max' => self::TERM_MAX_LENGTH, 'on' => 'list'),
            array('members', 'KeysStringValidator', 'className' => 'User', 'separator' => self::MEMBERS_SEPARATOR, 'on' => 'list'),
            array('members', 'required', 'on' => 'list'),
        );
        return CMap::mergeArray($base, $rules);
    }
    
    /**
     * Submits changes into database for create operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onCreate()
     */
    protected function onCreate()
    {
        $dao = new EventDAO('create', $this->loadRecord());
        $dao->attributes = $this->attributes;
        $dao->organizer = Yii::app()->organizer;
        return $dao->submit();
    }
    
    /**
     * Submits changes into database for update operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onUpdate()
     */
    protected function onUpdate()
    {
        $record = $this->loadRecord();
        $dao = new EventDAO('update', $record);
        $dao->attributes = $this->attributes;
        $dao->organizer = Yii::app()->organizer;
        return $dao->submit();
    }
    
    /**
     * Submits changes into database for delete operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onDelete()
     */
    protected function onDelete()
    {
        $record = $this->loadRecord();
        $dao = new EventDAO('delete', $record);
        $dao->organizer = Yii::app()->organizer;
        return $dao->submit();
    }
    
    
    /**
     * Fetchs api model's records list.
     * Used only for "list" API method
     * 
     * @return array records list
     */
    protected function fetchData()
    {
        $eventSearch = new EventSearch();
        $eventSearch->from = $this->from;
        $eventSearch->to = $this->to;
        $eventSearch->term = $this->term;
        $eventSearch->count = $this->count;
        $eventSearch->page = $this->page;
        $eventSearch->states = array_keys(EventStaff::getAvailableStates());
        $parser = new CQueryParser($this->members);
        $parser->separator = self::MEMBERS_SEPARATOR;
        $members = $parser->parse();
        $eventSearch->members = $members;
        $result = $eventSearch->search();
        return $result->getData();
    }

}