<?php
/**
 * PoiApiModel class
 * 
 * PHP version 5
 *  
 * @category ADPOI
 * @package  App.model
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
/**
 * PoiApiModel class
 *  * 
 * PoiApiModel is the model class for poi
 *  
 * @category ADPOI
 * @package  App.model
 * @author   Pohil Alexandr <pohil@5-soft.com>
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://adpoi.com/display/license proprietary
 * @link     http://adpoi.com/display/PhpDoc
 */
class PoiApiModel extends RestApiModel
{
    const TERM_MAX_LENGTH = 128;
    const DEFAULT_PAGE_SIZE = 5;
    const CATEGORIES_MAX_LENGTH = 128;
    const PRIORITIES_MAX_LENGTH = 128;
    
    /**
     * Poi's id
     * @var integer
     */
    public $id;
    
    /**
     * User id
     * @var string
     */
    public $user_id;
    
    /**
     * Poi's name
     * @var string
     */
    public $name;

    /**
     * Poi's description
     * @var string
     */
    public $description;
    
    /**
     * Poi's latitude
     * @var double
     */
    public $latitude;
    
    /**
     * Poi's longitude
     * @var double
     */
    public $longitude;
    
    /**
     * Poi's phone
     * @var sting
     */
    public $phone;
    
    /**
     * Poi's email
     * @var sting
     */
    public $email;
    
    /**
     * Skype of poi's manager
     * @var sting
     */
    public $skype;
    
    /**
     * Category name or comma separated categories names
     * @var string
     */
    public  $category;
    
    /**
     * Priority name or comma separated priorities names
     * @var string
     */
    public $priority;
    
    /**
     * Number of items per page
     * @var int
     */
    public $count;
    
    /**
     * Page number
     * @var int
     */
    public $page;
    
    /**
     * Seach radius
     * @var int
     */
    public $radius;
    
    /**
     * Search term
     * @var string
     */
    public $term;
    
    /**
     * Return the validation rulles
     * 
     * @return array validation rules to be applied 
     * when {@link validate()} is called.
     * @see CModel::rules()
     */
    public function rules()
    {
        $base = parent::rules();
        $rules = array(
            array('id', 'required', 'on' => 'update, delete, show'),
            array('id', 'numerical', 'integerOnly' => true, 'on' => 'update, delete, show'),
            array('id', 'exist', 'className' => 'Poi', 'attributeName' => 'id', 'on' => 'update, delete, show'),
            array('name, latitude, longitude, category', 'required', 'on' => 'create'),
            array('name', 'length', 'max' => Poi::NAME_MAX_LENGTH, 'on' => 'create, update'),
            array('category', 'length', 'max' => self::CATEGORIES_MAX_LENGTH, 'on' => 'create, update, list'),
            array('priority', 'length', 'max' => self::PRIORITIES_MAX_LENGTH, 'on' => 'list'),
            array('category', 'isCorrectCategories', 'on' => 'create, update, list', 'message' => Yii::t('api', 'Bad category value')),
            array('priority', 'isCorrectPriorities', 'on' => 'list', 'message' => Yii::t('api', 'Bad priority value')),
            array('latitude, ', 'numerical', 'min' => GGeoUtils::LATITUDE_MIN, 'max' => GGeoUtils::LATITUDE_MAX, 'on' => 'create, update, list'),
            array('longitude', 'numerical', 'min' => GGeoUtils::LONGITUDE_MIN, 'max' => GGeoUtils::LONGITUDE_MAX, 'on' => 'create, update, list'),
            array('phone, skype', 'length', 'max' => Poi::CONTACTS_MAX_LENGTH, 'allowEmpty' => false, 'on' => 'create, update'),
            array('email', 'length', 'max' => Poi::EMAIL_MAX_LENGH, 'allowEmpty' => false, 'on' => 'create, update'),
            array('email', 'email', 'on' => 'create, update'),
            array('description', 'safe', 'on' => 'create, update'),
            array('latitude, longitude, radius', 'required', 'on' => 'list'),
            array('term', 'length', 'max' => self::TERM_MAX_LENGTH, 'on' => 'list'),
            array('page', 'default', 'value' => 1, 'on' => 'list'),
            array('count', 'default', 'value' => self::DEFAULT_PAGE_SIZE, 'on' => 'list'),
        );
        return CMap::mergeArray($base, $rules);
    }
    
    /**
     * Submits changes into database for create operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onCreate()
     */
    protected function onCreate()
    {
        $record = $this->loadRecord();
        $poiCreate = new PoiDAO('create', $record);
        $poiCreate->attributes = $this->attributes;
        $poiCreate->category = $this->getCategoriesNames();
        if (!$poiCreate->submit()) {
            return false;
        }
        $this->id = $poiCreate->getObject()->id;
        return true;
    }
    
    /**
     * Submits changes into database for update operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onUpdate()
     */
    protected function onUpdate()
    {
        $poiCreate = new PoiDAO('update', $this->loadRecord());
        $poiCreate->attributes = $this->attributes;
        if (isset($this->category)) {
            $poiCreate->category = $this->getCategoriesNames();
        } 
        return $poiCreate->submit();
    }
    
    /**
     * Submits changes into database for delete operations
     *  
     * @return bool is operation success 
     * @see RestApiModel::onDelete()
     */
    protected function onDelete()
    {
        $record = $this->loadRecord();
        try {
            return $record->delete();
        } catch (CDbException $e) {
            $message = Yii::t('api', 'This poi connected with other object');
            throw new RestHttpException(RestHttpResponse::ERROR_CODE_INTERNAL_SERVER_ERROR, $message);
        }
        return false;
    }
    
    
    /**
     * Fetchs api model's records list.
     * Used only for "list" API method
     * 
     * @return array records list
     */
    protected function fetchData()
    {
        $poiSeach = new PoiSearch();
        $poiSeach->latitude = $this->latitude;
        $poiSeach->longitude = $this->longitude;
        $poiSeach->radius = $this->radius;
        $poiSeach->term = $this->term;
        $poiSeach->category = $this->isCorrectCategories('category', array());
        $poiSeach->priority = $this->isCorrectPriorities('priority', array());
        $poiSeach->user_id = $this->user_id;
        $poiSeach->count = $this->count;
        $poiSeach->page = $this->page;
        return $poiSeach->search();
    }
    
    /**
     * Validate function for checking category parameter
     * 
     * @param string $attribute variable name for validate
     * @param array  $params    additional parametrs for validate
     * 
     * @return array selected categories names
     */
    public function isCorrectCategories($attribute, $params)
    {
        $words = $this->getCategoriesNames();
        $diff = array_diff($words, PoiSearch::getAvailableCategories());
        if (!empty($diff)) {
            $this->addError($attribute, $params['message']);
        }
        return $words;
    }
    
    /**
     * Split the string of categories name to category word
     * 
     * @return array categoties name
     */
    public function getCategoriesNames()
    {
        $parser = new CQueryParser($this->category);
        $parser->separator = ',';
        $words = $parser->parse();
        return array_unique($words);
    }
    
    /**
     * Validate function for checking priority parameter
     * 
     * @param string $attribute variable name for validate
     * @param array  $params    additional parametrs for validate
     * 
     * @return array selected priorities names
     */
    public function isCorrectPriorities($attribute, $params)
    {
        $parser = new CQueryParser($this->priority);
        $parser->separator = ',';
        $words = $parser->parse();
        $diff = array_diff($words, PoiSearch::getAvailablePriorities());
        if (!empty($diff)) {
            $this->addError($attribute, $params['message']);
        }
        return $words;
    }
    
    /**
     * Returns primary key's data
     * 
     * @return array primary key's data
     * @see RestApiCreateMethod::getPrimaryData()
     */
    public function getPrimaryData()
    {
        $pk = array(
            'id' => $this->id,
        );
        return  $pk;
    }
}
