<?php
/**
 * PoiSearch class
 *
 * PHP version 5
 *
 * @category   ADPOI
 * @package    Core.model
 * @subpackage Core.model.search
 * @author     Pavel Fedulov <fedulov@5-soft.com>
 * @license    http://suppliers.com/display/license proprietary
 * @link       http://suppliers.com/display/PhpDoc
 */
/**
 * PoiSearch is the search model class
 * for searching POIs
 * 
 * @category   ADPOI
 * @package    Core.model
 * @subpackage Core.model.search
 * @author     Pavel Fedulov <fedulov@5-soft.com>
 * @license    http://suppliers.com/display/license proprietary
 * @link       http://suppliers.com/display/PhpDoc
 */
class PoiSearch extends SearchModel
{
    const DEFAULT_RADIUS = 5;
    const TERM_MAX_LENGTH = 512;
    
    const PRIORITY_FAVORITE = 'favorite';
    const PRIORITY_RECOMMENDATION = 'recommendation';
    const PRIORITY_RATING = 'rating';
    const PRIORITY_DISCOUNT = 'discount';
    
    const LOCATION_FILTER_ACCURACY = 'geo_distance';
    const LOCATION_FILTER_FAST = 'geo_distance_fast';
    const LOCATION_FILTER_FASTEST = 'geo_distance_fastest';
    const LOCATION_FILTER_WITHOUT_ROUTINES = 'without_stored_routine';
    
    /**
     * Object that should separated search term into pieces
     * @var CQueryParser
     */
    private $_queryParser;
    
    /**
     * Search term
     * @var string
     */
    public $term;
    
    /**
     * Search poi for specific user
     * @var int
     */
    public $user_id;
    
    /**
     * Search radius
     * @var int
     */
    public $radius;
    
    /**
     * Current latitude
     * @var double
     */
    public $latitude;
    
    /**
     * Current longitude
     * @var double
     */
    public $longitude;
    
    /**
     * Categories array
     * @var array
     */
    public $category = array();
    
    /**
     * Fetch POI's priority
     * @var array
     */
    public $priority = array();
    
    /**
     * Predefined POI identifiers array
     * @var array
     */
    public $ids;
    
    /**
     * Search location filtering routine name
     * @var string
     */
    public $locationFilter = self::LOCATION_FILTER_WITHOUT_ROUTINES;
    
    /**
     * (non-PHPdoc)
     * 
     * @return array validation rules
     * @see CModel::rules()
     */
    public function rules()
    {
        $base = parent::rules();
        $additional = array(
            array('latitude', 'numerical', 'min' => GGeoUtils::LATITUDE_MIN, 'max' => GGeoUtils::LATITUDE_MAX),
            array('longitude', 'numerical', 'min' => GGeoUtils::LONGITUDE_MIN, 'max' => GGeoUtils::LONGITUDE_MAX),
            array('user_id', 'numerical', 'integerOnly' => true, 'min' => 1),
            array('radius', 'numerical', 'integerOnly' => true),
            array('user_id', 'exist', 'className' => 'User', 'attributeName' => 'id'),
            array('ids', 'KeysArrayValidator', 'className' => 'Poi'),
            array('term', 'length', 'max' => self::TERM_MAX_LENGTH),
            array('radius', 'default', 'value' => self::DEFAULT_RADIUS),
            array('category', 'isValidCategory', 'message' => Yii::t('app', 'Bad category value.')),
            array('priority', 'default', 'value' => array(self::PRIORITY_FAVORITE, self::PRIORITY_RECOMMENDATION)),
            array('priority', 'isValidPriority', 'message' => Yii::t('app', 'Bad priority value.')),
        );
        return CMap::mergeArray($base, $additional);
    }
    
    /**
     * Validate function for checking category parameter
     * 
     * @param string $attribute variable name for validate
     * @param array  $params    additional parametrs for validate
     * 
     * @return void
     */
    public function isValidCategory($attribute, $params)
    {
        if (is_array($this->category)) {
            $diff = array_diff($this->category, self::getAvailableCategories());
            if (!empty($diff)) {
                $this->addError($attribute, $params['message']);
                return;
            }
        } else {
            $this->addError($attribute, $params['message']);
        }
    }
    
    /**
     * Validate function for checking priority parameter
     * 
     * @param string $attribute variable name for validate
     * @param array  $params    additional parametrs for validate
     * 
     * @return void
     */
    public function isValidPriority($attribute, $params)
    {
        if (is_array($this->priority)) {
            $diff = array_diff($this->priority, self::getAvailablePriorities());
            if (!empty($diff)) {
                $this->addError($attribute, $params['message']);
                return;
            }
        } else {
            $this->addError($attribute, $params['message']);
        }
    }
    
    /**
     * (non-PHPdoc)
     * 
     * @return string model class name
     * @see SearchModel::getModelClass()
     */
    public function getModelClass()
    {
        return 'PoiSearchItem';
    }
    
    /**
     * Returns initialized query parser object
     * 
     * @return CQueryParser query parser object
     */
    public function getQueryParser()
    {
        if (!isset($this->_queryParser)) {
            $this->_queryParser = new CQueryParser();
        }
        return $this->_queryParser;
    }
    
    /**
     * Parses term and returns term's words
     * 
     * @return array term's words
     */
    protected function parseTerm()
    {
        $parser = $this->getQueryParser();
        $parser->query = $this->term;
        return $parser->parse();
    }
    
    /**
     * Returns available category values
     * 
     * @return array category values
     */
    public static function getAvailableCategories()
    {
        $categories = Category::getList();
        return array_values(CHtml::listData($categories, 'id', 'api_name'));
    }
    
    /**
     * Returns available priority values
     * 
     * @return array priority values
     */
    public static function getAvailablePriorities()
    {
        return array(
            self::PRIORITY_FAVORITE,
            self::PRIORITY_RECOMMENDATION,
            self::PRIORITY_RATING,
            self::PRIORITY_DISCOUNT,
        );
    }
    
    /**
     * (non-PHPdoc)
     * 
     * @return CDataProvider configured data provider
     * @see SearchModel::build()
     */
    protected function build()
    {
        $criteria = new CDbCriteria;
        
        switch ($this->locationFilter) {
        case self::LOCATION_FILTER_ACCURACY:
        case self::LOCATION_FILTER_FAST:
            $criteria->select = array("*", "{$this->locationFilter}(t.latitude, t.longitude, :latitude, :longitude) * :earth_radius as distance");
            $params = array(
                ':latitude' => $this->latitude,
                ':longitude' => $this->longitude,
                ':earth_radius' => GGeoUtils::EARTH_RADIUS,
            );
            $criteria->addCondition('NOT(:earth_radius IS NULL) AND NOT(:latitude IS NULL) AND NOT(:longitude IS NULL)');
            $distanceCondition = "NOT(:earth_radius IS NULL) AND {$this->locationFilter}(t.latitude, t.longitude, :latitude, :longitude) <= :arc";
            break;
        case self::LOCATION_FILTER_FASTEST:
            $criteria->select = array("*", "{$this->locationFilter}(t.sin_latitude, :sin_latitude, t.cos_latitude, :cos_latitude, :longitude_radian - t.longitude_radian) * :earth_radius as distance");
            $latitudeRadian = GGeoUtils::degreeToRadian($this->latitude);
            $longitudeRadian = GGeoUtils::degreeToRadian($this->longitude);
            $params = array(
                ':sin_latitude' => sin($latitudeRadian),
                ':cos_latitude' => cos($latitudeRadian),
                ':longitude_radian' => $longitudeRadian,
                ':earth_radius' => GGeoUtils::EARTH_RADIUS,
            );
            $criteria->addCondition('NOT(:earth_radius IS NULL) AND NOT(:sin_latitude IS NULL) AND NOT(:cos_latitude IS NULL) AND NOT(:longitude_radian IS NULL)');
            $distanceCondition = "NOT(:earth_radius IS NULL) AND {$this->locationFilter}(t.sin_latitude, :sin_latitude, t.cos_latitude, :cos_latitude, :longitude_radian - t.longitude_radian) <= :arc";
            break;
        case self::LOCATION_FILTER_WITHOUT_ROUTINES:
            $criteria->select = array("*", "acos(t.sin_latitude * :sin_latitude + t.cos_latitude * :cos_latitude * cos(t.longitude_radian - :longitude_radian)) * :earth_radius as distance");
            $latitudeRadian = GGeoUtils::degreeToRadian($this->latitude);
            $longitudeRadian = GGeoUtils::degreeToRadian($this->longitude);
            $params = array(
                ':sin_latitude' => sin($latitudeRadian),
                ':cos_latitude' => cos($latitudeRadian),
                ':longitude_radian' => $longitudeRadian,
                ':earth_radius' => GGeoUtils::EARTH_RADIUS,
            );
            $criteria->addCondition('NOT(:earth_radius IS NULL) AND NOT(:sin_latitude IS NULL) AND NOT(:cos_latitude IS NULL) AND NOT(:longitude_radian IS NULL)');
            $distanceCondition = 'acos(t.sin_latitude * :sin_latitude + t.cos_latitude * :cos_latitude * cos(t.longitude_radian - :longitude_radian)) <= :arc';
            break;
        }
        $criteria->params = CMap::mergeArray($criteria->params, $params);
        
        if (isset($this->ids) && !empty($this->ids)) {
            $criteria->addInCondition('t.id', $this->ids);
        } else {
            $criteria->addCondition($distanceCondition);
            $criteria->params[':arc'] = $this->radius * GGeoUtils::METERS_IN_KILOMETER / GGeoUtils::EARTH_RADIUS;
            //$criteria->having = "distance <= :arc";
            
            $queryCriteria = new CDbCriteria;
            $terms = $this->parseTerm();
            foreach ($terms as $key => $value) {
                $termCriteria = new CDbCriteria;
                $termCriteria->compare('t.name', $value, true, 'OR');
                $termCriteria->compare('t.description', $value, true, 'OR');
                $queryCriteria->mergeWith($termCriteria);
            }
            $criteria->mergeWith($queryCriteria);
            
            $criteria->with = array('categories');
            $criteria->together = true;
            if (!empty($this->category)) {
                $criteria->compare('categories.id', Category::getIdsByNames($this->category));
            }
            
            $criteria->params = CMap::mergeArray($criteria->params, $params);
        }
        
        $criteria->group = 't.id';
        
        $config = array(
            'criteria' => $criteria,
        );
        return new CActiveDataProvider($this->getModelClass(), $config);
    }
    
    /**
     * Returns pagination config
     * 
     * @return array pagination config
     * @see SearchModel::getPagination()
     */
    protected function getPagination()
    {
        return !empty($this->ids) ? false : parent::getPagination();
    }
    
    /**
     * Applies sorting conditions for criteria
     * 
     * @param CDbCriteria $criteria search criteria
     * 
     * @return void
     * @see SearchModel::applySort()
     */
    protected function applySort($criteria)
    {
        $this->_applyPriority($criteria);
        $criteria->order .= empty($criteria->order) ? '' : ',';
        $criteria->order .= 'distance ASC';
    }
    
    /**
     * Applies priority filters to search criteria
     * 
     * @param CDbCriteria $criteria search criteria
     * 
     * @return void
     */
    private function _applyPriority($criteria)
    {
        foreach ($this->priority as $value) {
            switch ($value) {
            case self::PRIORITY_FAVORITE:
                $favoritePoiFilter = new FavoritePoiFilter();
                $favoritePoiFilter->user_id = $this->user_id;
                $favoritePoiFilter->apply($criteria);
                break;
            case self::PRIORITY_RECOMMENDATION:
                $recommendationPoiFilter = new RecommendationPoiFilter();
                $recommendationPoiFilter->to_id = $this->user_id;
                $recommendationPoiFilter->apply($criteria);
                break;
            case self::PRIORITY_RATING:
                $ratingPoiFilter = new RatingPoiFilter();
                $ratingPoiFilter->apply($criteria);
                break;
            case self::PRIORITY_DISCOUNT:
                break;
            }
        }
    }
}